@@ -1,12 +1,22 @@
+#' the response to a textDocument/signatureHelp Request
+#'
+#' If the symbol at the current position is a function, return its arguments
+#' (as with [base::args()]).
+#'
+#' @template id
+#' @template uri
+#' @template workspace
+#' @template document
+#' @template position
+#'
+#' @return a [Response] object
 signature_reply <- function(id, uri, workspace, document, position) {
-    line <- position$line
-    character <- position$character
 
-    if (!check_scope(uri, document, line)) {
+    if (!check_scope(uri, document, position)) {
         return(Response$new(id, list(signatures = NULL)))
     }
 
-    closure <- detect_closure(document, line, character)
+    closure <- detect_closure(document, position)
 
     SignatureInformation <- list()
     activeSignature <- -1
